# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawImdbClient, RawImdbClient
from .types.movie import Movie
from .types.movie_id import MovieId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ImdbClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawImdbClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawImdbClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawImdbClient
        """
        return self._raw_client

    def create_movie(
        self, *, title: str, rating: float, request_options: typing.Optional[RequestOptions] = None
    ) -> MovieId:
        """
        Add a movie to the database

        Parameters
        ----------
        title : str

        rating : float

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MovieId

        Examples
        --------
        from devintest import MyClientNameDevin

        client = MyClientNameDevin(
            base_url="https://yourhost.com/path/to/api",
        )
        client.imdb.create_movie(
            title="title",
            rating=1.1,
        )
        """
        _response = self._raw_client.create_movie(title=title, rating=rating, request_options=request_options)
        return _response.data

    def get_movie(self, id: MovieId, *, request_options: typing.Optional[RequestOptions] = None) -> Movie:
        """
        Retrieve a movie from the database based on the ID

        Parameters
        ----------
        id : MovieId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Movie

        Examples
        --------
        from devintest import MyClientNameDevin

        client = MyClientNameDevin(
            base_url="https://yourhost.com/path/to/api",
        )
        client.imdb.get_movie(
            id="tt0111161",
        )
        """
        _response = self._raw_client.get_movie(id, request_options=request_options)
        return _response.data


class AsyncImdbClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawImdbClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawImdbClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawImdbClient
        """
        return self._raw_client

    async def create_movie(
        self, *, title: str, rating: float, request_options: typing.Optional[RequestOptions] = None
    ) -> MovieId:
        """
        Add a movie to the database

        Parameters
        ----------
        title : str

        rating : float

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MovieId

        Examples
        --------
        import asyncio

        from devintest import AsyncMyClientNameDevin

        client = AsyncMyClientNameDevin(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.imdb.create_movie(
                title="title",
                rating=1.1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_movie(title=title, rating=rating, request_options=request_options)
        return _response.data

    async def get_movie(self, id: MovieId, *, request_options: typing.Optional[RequestOptions] = None) -> Movie:
        """
        Retrieve a movie from the database based on the ID

        Parameters
        ----------
        id : MovieId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Movie

        Examples
        --------
        import asyncio

        from devintest import AsyncMyClientNameDevin

        client = AsyncMyClientNameDevin(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.imdb.get_movie(
                id="tt0111161",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_movie(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from .errors.movie_does_not_exist_error import MovieDoesNotExistError
from .types.movie import Movie
from .types.movie_id import MovieId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawImdbClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_movie(
        self, *, title: str, rating: float, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[MovieId]:
        """
        Add a movie to the database

        Parameters
        ----------
        title : str

        rating : float

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MovieId]
        """
        _response = self._client_wrapper.httpx_client.request(
            "movies/create-movie",
            method="POST",
            json={
                "title": title,
                "rating": rating,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MovieId,
                    parse_obj_as(
                        type_=MovieId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_movie(self, id: MovieId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Movie]:
        """
        Retrieve a movie from the database based on the ID

        Parameters
        ----------
        id : MovieId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Movie]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"movies/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Movie,
                    parse_obj_as(
                        type_=Movie,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise MovieDoesNotExistError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        MovieId,
                        parse_obj_as(
                            type_=MovieId,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawImdbClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_movie(
        self, *, title: str, rating: float, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[MovieId]:
        """
        Add a movie to the database

        Parameters
        ----------
        title : str

        rating : float

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MovieId]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "movies/create-movie",
            method="POST",
            json={
                "title": title,
                "rating": rating,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MovieId,
                    parse_obj_as(
                        type_=MovieId,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_movie(
        self, id: MovieId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Movie]:
        """
        Retrieve a movie from the database based on the ID

        Parameters
        ----------
        id : MovieId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Movie]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"movies/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Movie,
                    parse_obj_as(
                        type_=Movie,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise MovieDoesNotExistError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        MovieId,
                        parse_obj_as(
                            type_=MovieId,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper

if typing.TYPE_CHECKING:
    from .imdb.client import AsyncImdbClient, ImdbClient


class MyClientNameDevin:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : str
        The base url to use for requests from the client.

    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from devintest import MyClientNameDevin

    client = MyClientNameDevin(
        base_url="https://yourhost.com/path/to/api",
    )
    """

    def __init__(
        self,
        *,
        base_url: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=base_url,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._imdb: typing.Optional[ImdbClient] = None

    @property
    def imdb(self):
        if self._imdb is None:
            from .imdb.client import ImdbClient  # noqa: E402

            self._imdb = ImdbClient(client_wrapper=self._client_wrapper)
        return self._imdb


class AsyncMyClientNameDevin:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : str
        The base url to use for requests from the client.

    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from devintest import AsyncMyClientNameDevin

    client = AsyncMyClientNameDevin(
        base_url="https://yourhost.com/path/to/api",
    )
    """

    def __init__(
        self,
        *,
        base_url: str,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=base_url,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._imdb: typing.Optional[AsyncImdbClient] = None

    @property
    def imdb(self):
        if self._imdb is None:
            from .imdb.client import AsyncImdbClient  # noqa: E402

            self._imdb = AsyncImdbClient(client_wrapper=self._client_wrapper)
        return self._imdb
